import { HttpStatusCode } from 'axios';
import { drugsFixture } from '@/models/fixtures/drugFixtures';
import { mockNetworkResponse } from '@/utils/mockNetworkResponse';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { apiPath } from '@/redux/apiPath';
import { getDrugs } from './drugs.thunks';
import drugsReducer from './drugs.slice';
import { DrugsState } from './drugs.types';

const mockedAxiosClient = mockNetworkResponse();
const SEARCH_QUERY = 'aspirin';

const INITIAL_STATE: DrugsState = {
  data: [],
  loading: 'idle',
  error: { name: '', message: '' },
};

describe('drugs reducer', () => {
  let store = {} as ToolkitStoreWithSingleSlice<DrugsState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('drugs', drugsReducer);
  });

  it('should match initial state', () => {
    const action = { type: 'unknown' };

    expect(drugsReducer(undefined, action)).toEqual(INITIAL_STATE);
  });
  it('should update store after succesfull getDrugs query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getDrugsStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.Ok, drugsFixture);

    const { type } = await store.dispatch(getDrugs(SEARCH_QUERY));
    const { data, loading, error } = store.getState().drugs;

    expect(type).toBe('project/getDrugs/fulfilled');
    expect(loading).toEqual('succeeded');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual(drugsFixture);
  });

  it('should update store after failed getDrugs query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getDrugsStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.NotFound, []);

    const { type } = await store.dispatch(getDrugs(SEARCH_QUERY));
    const { data, loading, error } = store.getState().drugs;

    expect(type).toBe('project/getDrugs/rejected');
    expect(loading).toEqual('failed');
    expect(error).toEqual({ message: '', name: '' });
    expect(data).toEqual([]);
  });

  it('should update store on loading getDrugs query', async () => {
    mockedAxiosClient
      .onGet(apiPath.getDrugsStringWithQuery(SEARCH_QUERY))
      .reply(HttpStatusCode.Ok, drugsFixture);

    const drugsPromise = store.dispatch(getDrugs(SEARCH_QUERY));

    const { data, loading } = store.getState().drugs;
    expect(data).toEqual([]);
    expect(loading).toEqual('pending');

    drugsPromise.then(() => {
      const { data: dataPromiseFulfilled, loading: promiseFulfilled } = store.getState().drugs;

      expect(dataPromiseFulfilled).toEqual(drugsFixture);
      expect(promiseFulfilled).toEqual('succeeded');
    });
  });
});
